#define DS_pin PF_IDR_IDR4
#define DS_out PF_ODR_ODR4
#define DS_dir PF_DDR_DDR4

#define pin_free  do{DS_dir=0;          }while(0)
#define pin_dn    do{DS_out=0; DS_dir=1;}while(0) 
#define pin_up    do{DS_out=1; DS_dir=1;}while(0)

#define SKIP  0xcc
#define RUN   0x44
#define READ  0xbe
#define WRITE 0x4e

struct DS18_spad
{
 u8 T_lsb;
 u8 T_msb;
 u8 Th;
 u8 Tl;
 u8 conf;
 u8 na0;
 u8 na1;
 u8 na2;
 u8 crc;
} spad;

bool DS18_reset ()
{
 bool retval=0;
 
 asm ("sim");
 pin_dn;
 delay_us(500);
 pin_free;
 delay_us(65);
 if(!DS_pin)if(!DS_pin)retval=1; 
 asm ("rim");
 delay_us(480-65);
 
 return retval; 
}

u8 read_byte (void)
{
 u8 i;
 u8 result=0; 

 pin_free;
 
 for(i=0;i<8;i++) 
 {    
  delay_us(60);
  asm ("sim");
  pin_dn;
  delay_us(5);        
  pin_free;
  delay_us(10);        
  result = result >> 1;    
  if(DS_pin) result |= 0x80;
  asm ("rim");
  delay_us(45);
 } 
 return result;
}

void write_byte(u8 data)
{
 u8 i;
 
 pin_free;

 for(i=0;i<8;i++)
 {   
  delay_us(60);
  asm ("sim");
  pin_dn;
  delay_us(5);      
  if(data & 1)pin_up;
  else        pin_dn;
  data = data >> 1; 
  asm ("rim");
  delay_us(55); 
  pin_free;
 } 
}

bool do_conv (void)
{  
 DS18_reset();   
 write_byte(SKIP);     
 write_byte(RUN); 
 pin_up;
 delay_ms(1000); 
 return DS18_reset();
}

s16 read_short()
{
 s16 rval;    
        
 write_byte(SKIP);     
 write_byte(READ);     

 rval = read_byte();
 rval|= read_byte()<<8; 

 DS18_reset();
  
 return rval*10>>4;
}

struct DS18_spad* read_spad (struct DS18_spad* newst)
{
 u8 i;
 
 write_byte(SKIP);     
 write_byte(READ);
   
 for(i=0;i<9;i++)*((u8*)newst+i) = read_byte();
      
 return newst;
}

bool check_crc (u8* spad)
{
 #define CRC8INIT   0x00
 #define CRC8POLY   0x18  //0x18 = X^8+X^5+X^4+X^0, 0b00011000
 
 u8 crc=CRC8INIT;
 u8 i;

 for(i=0;i<9;i++)
 {
  u8 bit_counter = 8;
  u8 data = *(spad+i);
  bool feedback;

  while(bit_counter--)
  {
   feedback = (crc ^ data) & 0x01;
   if (feedback) crc = crc ^ CRC8POLY;
   crc >>= 1;  
   data>>= 1;   
   if (feedback) crc = crc | 0x80;
  }
 }  
 
 if(crc)return 0;
 return 1; 
}

