/********************************************************
Workspace: IAR+SPL
MCU      : STM8S003k3 (16MHz HSI)
Project  : freeser_relay_2
Version  : 0
Date     : Sept, 2017
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include "bitset.c"
#include "defines.c"
#include "periph_init.c"
#include "delay_routines.c"
#include "1wire_routines.c"
#include "adc_routines.c"
#include "7seg_routines.c"

volatile u16 tim0,tim1,tim2,tim3;
volatile bool iwdg_t4;

extern "C"
{
 INTERRUPT_HANDLER(TIM4_OVF, 23)   //1ms
 {
  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  if(tim3)tim3--;  
  display_data ();
  
  iwdg_t4=1;
  TIM4->SR1 = 0;
 }
}

void devcon();

void main()
{ 
 cfg.gpio();
 cfg.clk();
 cfg.ee(); 
 cfg.t1();
 cfg.t4();
 cfg.adc();

 detect_7seg_polarity(ss_pol);
 
#ifndef debug 
 cfg.iwdg();
#endif

 asm("rim");
 
 endless
 {
  devcon();
  
  if(iwdg_t4){iwdg_t4=0;IWDG_ReloadCounter();}
 }
}

void devcon()
{
 static bool itr=0;
 static enum {start,work,err}dev_st = start; 
 static enum {no_err, ds18_lost, crc_err, no_t_reach}errnum = no_err;  
 static bool adc_result_ready=0; 
 #define RSN 3
 static s16 t_result[RSN]; 
 static s16 t_set; 
 static u8 i; 
 s16 stmp;
 u8 tmp; 

 //     
 // t_set = t*10       
 if(read_adc(&tmp))
 {
  adc_result_ready=1;
  t_set = top_t;
  t_set+= btm_t; 
  t_set = (u32)t_set*tmp>>8;   
  t_set = t_set-btm_t;  
 }   
 
 //      (     )  
 if(!tim0)
 {  
  static u8 crc_err_cnt;    

#ifdef debug 
  tim0=(u16)6E3; //  
#else        
  tim0=(u16)60E3;//  
#endif
   
  do_conv();  
  read_spad(&spad);   
  
  //          
  if(!check_crc ((u8*)&spad))
  {
   if(crc_err_cnt<10)crc_err_cnt++;
   else errnum = crc_err;
  }   
  else
  { 
   u8 i; 
   if(crc_err_cnt)crc_err_cnt--;
  
   for(i=RSN;i-1;i--)t_result[i-1]=t_result[i-2];
  
   t_result[0] = spad.T_lsb;
   t_result[0]|= spad.T_msb<<8;
   t_result[0] = t_result[0]*10>>4; // *10
   //      
   // 
   if(t_result[0]>600 || t_result[0]<-300) t_result[0]=t_result[1];
  } 
 } 

 if(errnum!=no_err){dev_st=err;itr=0;}

 switch (dev_st)
 {
  case start:
  { 
   if(!itr)
   {
    itr=1;
    i=5; 
    tim0=0; 
    MOTOR_OFF;
   } 
   
   //  
   if(!tim1)
   {
    tim1=1000;
    dpp[0]^=1; 
    //       
    if(dpp[0] && adc_result_ready)
    { 
     adc_result_ready=0; 
     stmp=t_set/10;
     
     if(stmp<0)
     {
      stmp=-stmp;
      dbuf[1]='-'; 
      dbuf[0]=stmp; 
     }
     else dbuf[1]=' ';
     dbuf[0]=stmp;
    }
    //    
    else
    {   
     stmp=t_result[0]/10;  
     if(stmp<0)
     {   
      stmp=-stmp;
      convert_data(stmp);
      dbuf[1]='-'; 
     }
     else
     { 
      convert_data(stmp); 
      if(stmp<10)dbuf[1]=' ';
     }    
    }     
   }
   //     (5 )
   if(!tim2)
   { 
   #ifdef debug 
    if(i--)tim2=(u16)1000; else dev_st = work;//  ,  5
   #else        
    if(i--)tim2=(u16)6E4;  else dev_st = work; //  ,  5
   #endif    
   }
      
   if(dev_st!=start)
   {
    itr=0;
    dpp[0]=0; 
   }
  }
  break;

  case work:
  { 
   if(!itr)
   {
    itr=1;
    i=0; 
    tim1=0;
    tim2=0;
   }    
   
   //  
   if(!tim2)
   { 
    static bool m_on_trg; 
    static bool m_off_trg;   
   
    tim2 = (u16)60E3;  
    
    //            
    //     
    static u8 cutoff_cntdn; 
    //           
    static u8 m_cntdn; 
    
    //       
    //         
    if(t_set/10<t_result[0]/10 && !m_cntdn && !m_on_trg)
    {  
     m_on_trg = 1; 
     m_off_trg = 0; 
     
     //      
     cutoff_cntdn = 0xff;
     MOTOR_ON; 
     //        
     dpp[0]=1;     
    }
    //     
    else if(t_set/10>t_result[0]/10) 
    {
     MOTOR_OFF;
     dpp[0]=0;
   
     //          
     if(!m_off_trg)
     {
      m_off_trg = 1;
      //      TIMEWAIT 
      m_cntdn   = TIMEWAIT;
     }
    } 
    
    //     (  )
    if(m_cntdn)m_cntdn--;
    else       m_on_trg = 0;
    
    //      
    //     (  )   
    if(!m_off_trg && t_set/10<t_result[0]/10)
    {
     if(cutoff_cntdn)cutoff_cntdn--;  
     else errnum = no_t_reach;
    } 
    
   } 

   /*      */  
   {   
   static s16 t_set_tmp;
   stmp = t_set/10;   
   //   
   if(t_set_tmp != stmp)
   {
    t_set_tmp = stmp; 
    //       
    tim1 = 5000;
   }    
     
   //      
   //    
   //            
   //  
   static bool rec_trg; 
   //5         
   if(tim1)
   {          
    rec_trg=1;
    stmp = t_set_tmp;
    if(stmp<0)
    {
     stmp=-stmp;
     dbuf[1]='-'; 
     dbuf[0]=stmp; 
    }
    else dbuf[1]=' ';
    dbuf[0]=stmp;
   } 
   //          
   else
   {       
    static s8 t_result_tmp; 
        
    //        
    stmp=t_result[0]/10;  
    if(t_result_tmp != stmp || rec_trg)
    {  
     rec_trg = 0;
     t_result_tmp = stmp;
     if(stmp<0)
     {   
      stmp=-stmp;
      convert_data(stmp);
      dbuf[1]='-'; 
     }
     else
     { 
      convert_data(stmp); 
      if(stmp<10)dbuf[1]=' ';
     } 
    }
   }   
   } 
   

   if(dev_st!=work){itr=0;}
  }
  break;
  
  case err:
  {
   if(!itr)
   {
    itr=1; 
    MOTOR_OFF; 
    dpp[0]=0;
    dbuf[1]='E';
    if(errnum==crc_err)        dbuf[0]=0;
    else if(errnum==no_t_reach)dbuf[0]=1;
   } 
   
   if(dev_st!=err){itr=0; }
  }
  break; 

  default:break;
 }

}
